import warnings
from ovos_solver_openai_persona.engines import OpenAIChatCompletionsSolver

class OpenAIPersonaSolver(OpenAIChatCompletionsSolver):
    def __init__(self, *args, **kwargs):
           """
           Initializes the solver and issues a deprecation warning.
           
           A DeprecationWarning is raised advising to use OpenAIChatCompletionsSolver instead.
           """
           warnings.warn(
              "use OpenAIChatCompletionsSolver instead",
              DeprecationWarning,
              stacklevel=2,
           )
           super().__init__(*args, **kwargs)

# for ovos-persona
LLAMA_DEMO = {
  "name": "Remote LLama",
  "solvers": [
    "ovos-solver-openai-plugin"
  ],
  "ovos-solver-openai-plugin": {
    "api_url": "https://llama.smartgic.io/v1",
    "key": "sk-xxxx",
    "model": "llama3.1:8b"
  }
}

if __name__ == "__main__":
    bot = OpenAIChatCompletionsSolver(LLAMA_DEMO["ovos-solver-openai-plugin"])
    #for utt in bot.stream_utterances("describe quantum mechanics in simple terms"):
    #    print(utt)
        #  Quantum mechanics is a branch of physics that studies the behavior of atoms and particles at the smallest scales.
        #  It describes how these particles interact with each other, move, and change energy levels.
        #  Think of it like playing with toy building blocks that represent particles.
        #  Instead of rigid structures, these particles can be in different energy levels or "states." Quantum mechanics helps scientists understand and predict these states, making it crucial for many fields like chemistry, materials science, and engineering.

    # Quantum mechanics is a branch of physics that deals with the behavior of particles on a very small scale, such as atoms and subatomic particles. It explores the idea that particles can exist in multiple states at once and that their behavior is not predictable in the traditional sense.
    print(bot.spoken_answer("what is the definition of computer", lang="en-US"))
    # O português Pedro Álvares Cabral encontrou o caminho marítimo para o Brasil em 1500. Ele foi o responsável por descobrir o litoral brasileiro, embora Cristóvão Colombo tenha chegado à América do Sul em 1498, cinco anos antes. Cabral desembarcou na atual costa de Alagoas, no Nordeste do Brasil.



from typing import Tuple

from ovos_plugin_manager.templates.transformers import DialogTransformer

from ovos_solver_openai_persona import OpenAIChatCompletionsSolver


class OpenAIDialogTransformer(DialogTransformer):
    def __init__(self, name="ovos-dialog-transformer-openai-plugin", priority=10, config=None):
        """
        Initializes the OpenAIDialogTransformer with a name, priority, and configuration.
        
        Creates an OpenAIChatCompletionsSolver using the provided API key, API URL, and a system prompt from the configuration or a default prompt if not specified.
        """
        super().__init__(name, priority, config)
        self.solver = OpenAIChatCompletionsSolver({
            "key": self.config.get("key"),
            'api_url': self.config.get('api_url', 'https://api.openai.com/v1'),
            "enable_memory": False,
            "system_prompt": self.config.get("system_prompt") or "Your task is to rewrite text as if it was spoken by a different character"
        })

    def transform(self, dialog: str, context: dict = None) -> Tuple[str, dict]:
        """
        Transforms the dialog string using a character-specific prompt if available.
        
        If a prompt is provided in the context or configuration, rewrites the dialog as if spoken by a different character using the solver; otherwise, returns the original dialog unchanged.
        
        Args:
            dialog: The dialog string to be transformed.
            context: Optional dictionary containing transformation context, such as a prompt or language.
        
        Returns:
            A tuple containing the transformed (or original) dialog and the unchanged context.
        """
        prompt = context.get("prompt") or self.config.get("rewrite_prompt")
        if not prompt:
            return dialog, context
        return self.solver.get_spoken_answer(f"{prompt} : {dialog}", lang=context.get("lang")), context



from ovos_solver_openai_persona.engines import OpenAIChatCompletionsSolver
from ovos_plugin_manager.templates.solvers import TldrSolver
from ovos_plugin_manager.templates.language import LanguageTranslator, LanguageDetector
from typing import Dict, Optional


class OpenAISummarizer(TldrSolver):
    TEMPLATE = """Your task is to summarize the text into a suitable format.
Answer in plaintext with no formatting, 2 paragraphs long at most. 
Focus on the most important information.
---------------------
{content}
"""
    def __init__(self, config: Optional[Dict] = None,
                 translator: Optional[LanguageTranslator] = None,
                 detector: Optional[LanguageDetector] = None,
                 priority: int = 50,
                 enable_tx: bool = False,
                 enable_cache: bool = False,
                 internal_lang: Optional[str] = None):
        super().__init__(config=config, translator=translator,
                         detector=detector, priority=priority,
                         enable_tx=enable_tx, enable_cache=enable_cache,
                         internal_lang=internal_lang)
        self.llm = OpenAIChatCompletionsSolver(config=config, translator=translator,
                                               detector=detector, priority=priority,
                                               enable_tx=enable_tx, enable_cache=enable_cache,
                                               internal_lang=internal_lang)
        self.prompt_template = self.config.get("prompt_template") or self.TEMPLATE


    def get_tldr(self, document: str, lang: Optional[str] = None) -> str:
        """
        Summarize the provided document.

        :param document: The text of the document to summarize, assured to be in the default language.
        :param lang: Optional language code.
        :return: A summary of the provided document.
        """
        prompt = self.prompt_template.format(content=document)
        return self.llm.get_spoken_answer(prompt, lang)



#!/usr/bin/env python3
import os

from setuptools import setup

BASEDIR = os.path.abspath(os.path.dirname(__file__))


def required(requirements_file):
    """ Read requirements file and remove comments and empty lines. """
    with open(os.path.join(BASEDIR, requirements_file), 'r') as f:
        requirements = f.read().splitlines()
        if 'MYCROFT_LOOSE_REQUIREMENTS' in os.environ:
            print('USING LOOSE REQUIREMENTS!')
            requirements = [r.replace('==', '>=').replace('~=', '>=') for r in requirements]
        return [pkg for pkg in requirements
                if pkg.strip() and not pkg.startswith("#")]


with open(os.path.join(BASEDIR, "README.md"), "r") as f:
    long_description = f.read()



def get_version():
    """ Find the version of the package"""
    version = None
    version_file = os.path.join(BASEDIR, 'ovos_solver_openai_persona', 'version.py')
    major, minor, build, alpha = (None, None, None, None)
    with open(version_file) as f:
        for line in f:
            if 'VERSION_MAJOR' in line:
                major = line.split('=')[1].strip()
            elif 'VERSION_MINOR' in line:
                minor = line.split('=')[1].strip()
            elif 'VERSION_BUILD' in line:
                build = line.split('=')[1].strip()
            elif 'VERSION_ALPHA' in line:
                alpha = line.split('=')[1].strip()

            if ((major and minor and build and alpha) or
                    '# END_VERSION_BLOCK' in line):
                break
    version = f"{major}.{minor}.{build}"
    if alpha and int(alpha) > 0:
        version += f"a{alpha}"
    return version


PERSONA_ENTRY_POINT = 'Remote Llama=ovos_solver_openai_persona:LLAMA_DEMO'
PLUGIN_ENTRY_POINT = 'ovos-solver-openai-plugin=ovos_solver_openai_persona.engines:OpenAIChatCompletionsSolver'
DIALOG_PLUGIN_ENTRY_POINT = 'ovos-dialog-transformer-openai-plugin=ovos_solver_openai_persona.dialog_transformers:OpenAIDialogTransformer'
SUMMARIZER_ENTRY_POINT = 'ovos-summarizer-openai-plugin=ovos_solver_openai_persona.summarizer:OpenAISummarizer'


setup(
    name='ovos-openai-plugin',
    version=get_version(),
    description='A question solver plugin for ovos',
    url='https://github.com/OpenVoiceOS/ovos-solver-plugin-openai-persona',
    author='jarbasai',
    author_email='jarbasai@mailfence.com',
    license='MIT',
    packages=['ovos_solver_openai_persona'],
    zip_safe=True,
    keywords='ovos plugin utterance fallback query',
    entry_points={
        'neon.plugin.solver': PLUGIN_ENTRY_POINT,
        "opm.transformer.dialog": DIALOG_PLUGIN_ENTRY_POINT,
        'opm.solver.summarization': SUMMARIZER_ENTRY_POINT,
        "opm.plugin.persona": PERSONA_ENTRY_POINT
    },
    install_requires=required("requirements.txt"),
    long_description=long_description,
    long_description_content_type='text/markdown'
)
